@@ -78,7 +78,7 @@ define('NET_SIEVE_STATE_TRANSACTION', 3);
  * @copyright 2002-2003 Richard Heyes
  * @copyright 2006-2008 Anish Mistry
  * @license   http://www.opensource.org/licenses/bsd-license.php BSD
- * @version   Release: 1.4.6
+ * @version   Release: 1.4.8
  * @link      http://pear.php.net/package/Net_Sieve
  * @link      http://tools.ietf.org/html/rfc5228 RFC 5228 (Sieve: An Email
  *            Filtering Language)
@@ -94,27 +94,28 @@ class Net_Sieve
      *
      * @var array
      */
-    var $supportedAuthMethods = array(
+    var $supportedAuthMethods = [
         'DIGEST-MD5',
         'CRAM-MD5',
         'EXTERNAL',
         'PLAIN' ,
         'LOGIN',
         'GSSAPI',
-        'XOAUTH2'
-    );
+        'XOAUTH2',
+        'OAUTHBEARER'
+    ];
 
     /**
      * SASL authentication methods that require Auth_SASL.
      *
      * @var array
      */
-    var $supportedSASLAuthMethods = array('DIGEST-MD5', 'CRAM-MD5');
+    var $supportedSASLAuthMethods = ['DIGEST-MD5', 'CRAM-MD5'];
 
     /**
      * The socket handle.
      *
-     * @var resource
+     * @var Net_Socket
      */
     var $_sock;
 
@@ -130,35 +131,35 @@ class Net_Sieve
      *
      * @var array
      */
-    var $_capability = array();
+    var $_capability = [];
 
     /**
      * Current state of the connection.
      *
      * One of the NET_SIEVE_STATE_* constants.
      *
-     * @var integer
+     * @var int
      */
     var $_state;
 
     /**
      * PEAR object to avoid strict warnings.
      *
-     * @var PEAR_Error
+     * @var PEAR
      */
     var $_pear;
 
     /**
      * Constructor error.
      *
-     * @var PEAR_Error
+     * @var PEAR_Error|null
      */
-    var $_error;
+    var $_error = null;
 
     /**
      * Whether to enable debugging.
      *
-     * @var boolean
+     * @var bool
      */
     var $_debug = false;
 
@@ -174,14 +175,14 @@ class Net_Sieve
     /**
      * Whether to pick up an already established connection.
      *
-     * @var boolean
+     * @var bool
      */
     var $_bypassAuth = false;
 
     /**
      * Whether to use TLS if available.
      *
-     * @var boolean
+     * @var bool
      */
     var $_useTLS = true;
 
@@ -195,7 +196,7 @@ class Net_Sieve
     /**
      * Maximum number of referral loops
      *
-     * @var array
+     * @var int
      */
     var $_maxReferralCount = 15;
 
@@ -223,15 +224,17 @@ class Net_Sieve
      * @param string  $user       Login username.
      * @param string  $pass       Login password.
      * @param string  $host       Hostname of server.
-     * @param string  $port       Port of server.
+     * @param int     $port       Port of server.
      * @param string  $logintype  Type of login to perform (see
-     *                            $supportedAuthMethods).
+     *                            $supportedAuthMethods), use `OAUTH` and lib
+     *                            will choose between OAUTHBEARER or XOAUTH2
+     *                            according the server's capabilities.
      * @param string  $euser      Effective user. If authenticating as an
      *                            administrator, login as this user.
-     * @param boolean $debug      Whether to enable debugging (@see setDebug()).
-     * @param string  $bypassAuth Skip the authentication phase. Useful if the
+     * @param bool    $debug      Whether to enable debugging (@see setDebug()).
+     * @param bool    $bypassAuth Skip the authentication phase. Useful if the
      *                            socket is already open.
-     * @param boolean $useTLS     Use TLS if available.
+     * @param bool    $useTLS     Use TLS if available.
      * @param array   $options    Additional options for
      *                            stream_context_create().
      * @param mixed   $handler    A callback handler for the debug output.
@@ -274,14 +277,17 @@ class Net_Sieve
         }
 
         if (is_string($user) && strlen($user) && strlen($pass)) {
-            $this->_error = $this->_handleConnectAndLogin();
+            $res = $this->_handleConnectAndLogin();
+            if ($res !== true) {
+                $this->_error = $res;
+            }
         }
     }
 
     /**
      * Returns any error that may have been generated in the constructor.
      *
-     * @return boolean|PEAR_Error  False if no error, PEAR_Error otherwise.
+     * @return bool|PEAR_Error False if no error, PEAR_Error otherwise.
      */
     function getError()
     {
@@ -291,8 +297,8 @@ class Net_Sieve
     /**
      * Sets the debug state and handler function.
      *
-     * @param boolean $debug   Whether to enable debugging.
-     * @param string  $handler A custom debug handler. Must be a valid callback.
+     * @param bool   $debug   Whether to enable debugging.
+     * @param string $handler A custom debug handler. Must be a valid callback.
      *
      * @return void
      */
@@ -331,7 +337,7 @@ class Net_Sieve
     /**
      * Connects to the server and logs in.
      *
-     * @return boolean  True on success, PEAR_Error on failure.
+     * @return bool|PEAR_Error True on success, PEAR_Error on failure.
      */
     function _handleConnectAndLogin()
     {
@@ -357,9 +363,9 @@ class Net_Sieve
      * @param string  $port    Port of server.
      * @param array   $options List of options to pass to
      *                         stream_context_create().
-     * @param boolean $useTLS  Use TLS if available.
+     * @param bool    $useTLS  Use TLS if available.
      *
-     * @return boolean  True on success, PEAR_Error otherwise.
+     * @return bool|PEAR_Error True on success, PEAR_Error otherwise.
      */
     function connect($host, $port, $options = null, $useTLS = true)
     {
@@ -423,10 +429,10 @@ class Net_Sieve
     /**
      * Disconnect from the Sieve server.
      *
-     * @param boolean $sendLogoutCMD Whether to send LOGOUT command before
-     *                               disconnecting.
+     * @param bool $sendLogoutCMD Whether to send LOGOUT command before
+     *                            disconnecting.
      *
-     * @return boolean  True on success, PEAR_Error otherwise.
+     * @return bool|PEAR_Error True on success, PEAR_Error otherwise.
      */
     function disconnect($sendLogoutCMD = true)
     {
@@ -440,9 +446,9 @@ class Net_Sieve
      * @param string  $pass       Login password.
      * @param string  $logintype  Type of login method to use.
      * @param string  $euser      Effective UID (perform on behalf of $euser).
-     * @param boolean $bypassAuth Do not perform authentication.
+     * @param bool    $bypassAuth Do not perform authentication.
      *
-     * @return boolean  True on success, PEAR_Error otherwise.
+     * @return bool|PEAR_Error True on success, PEAR_Error otherwise.
      */
     function login($user, $pass, $logintype = null, $euser = '', $bypassAuth = false)
     {
@@ -456,7 +462,7 @@ class Net_Sieve
             return $this->_pear->raiseError('Not currently in AUTHORISATION state', 1);
         }
 
-        if (!$bypassAuth ) {
+        if (!$bypassAuth) {
             $res = $this->_cmdAuthenticate($user, $pass, $logintype, $euser);
             if (is_a($res, 'PEAR_Error')) {
                 return $res;
@@ -473,7 +479,7 @@ class Net_Sieve
      *
      * @param string $active Will be set to the name of the active script
      *
-     * @return array  Indexed array of scriptnames, PEAR_Error on failure
+     * @return array|PEAR_Error Indexed array of scriptnames, PEAR_Error on failure
      */
     function listScripts(&$active = null)
     {
@@ -491,13 +497,15 @@ class Net_Sieve
     /**
      * Returns the active script.
      *
-     * @return string  The active scriptname.
+     * @return string|null The active scriptname.
      */
     function getActive()
     {
         if (is_array($scripts = $this->_cmdListScripts())) {
-            return $scripts[1];
+            return isset($scripts[1]) ? $scripts[1] : null;
         }
+
+        return null;
     }
 
     /**
@@ -505,7 +513,7 @@ class Net_Sieve
      *
      * @param string $scriptname The name of the script to be set as active.
      *
-     * @return boolean  True on success, PEAR_Error on failure.
+     * @return bool|PEAR_Error True on success, PEAR_Error on failure.
      */
     function setActive($scriptname)
     {
@@ -517,7 +525,7 @@ class Net_Sieve
      *
      * @param string $scriptname The name of the script to be retrieved.
      *
-     * @return string  The script on success, PEAR_Error on failure.
+     * @return string|PEAR_Error The script on success, PEAR_Error on failure.
      */
     function getScript($scriptname)
     {
@@ -529,9 +537,9 @@ class Net_Sieve
      *
      * @param string  $scriptname Name of the script.
      * @param string  $script     The script content.
-     * @param boolean $makeactive Whether to make this the active script.
+     * @param bool    $makeactive Whether to make this the active script.
      *
-     * @return boolean  True on success, PEAR_Error on failure.
+     * @return bool|PEAR_Error True on success, PEAR_Error on failure.
      */
     function installScript($scriptname, $script, $makeactive = false)
     {
@@ -552,7 +560,7 @@ class Net_Sieve
      *
      * @param string $scriptname Name of the script.
      *
-     * @return boolean  True on success, PEAR_Error on failure.
+     * @return bool|PEAR_Error True on success, PEAR_Error on failure.
      */
     function removeScript($scriptname)
     {
@@ -562,10 +570,10 @@ class Net_Sieve
     /**
      * Checks if the server has space to store the script by the server.
      *
-     * @param string  $scriptname The name of the script to mark as active.
-     * @param integer $size       The size of the script.
+     * @param string $scriptname The name of the script to mark as active.
+     * @param int    $size       The size of the script.
      *
-     * @return boolean|PEAR_Error  True if there is space, PEAR_Error otherwise.
+     * @return bool|PEAR_Error True if there is space, PEAR_Error otherwise.
      *
      * @todo Rename to hasSpace()
      */
@@ -586,7 +594,7 @@ class Net_Sieve
     /**
      * Returns the list of extensions the server supports.
      *
-     * @return array  List of extensions or PEAR_Error on failure.
+     * @return array|PEAR_Error List of extensions or PEAR_Error on failure.
      */
     function getExtensions()
     {
@@ -602,8 +610,7 @@ class Net_Sieve
      *
      * @param string $extension The extension to check.
      *
-     * @return boolean  Whether the extension is supported or PEAR_Error on
-     *                  failure.
+     * @return bool|PEAR_Error Whether the extension is supported or PEAR_Error on failure.
      */
     function hasExtension($extension)
     {
@@ -626,7 +633,7 @@ class Net_Sieve
     /**
      * Returns the list of authentication methods the server supports.
      *
-     * @return array  List of authentication methods or PEAR_Error on failure.
+     * @return array|PEAR_Error List of authentication methods or PEAR_Error on failure.
      */
     function getAuthMechs()
     {
@@ -642,8 +649,7 @@ class Net_Sieve
      *
      * @param string $method The method to check.
      *
-     * @return boolean  Whether the method is supported or PEAR_Error on
-     *                  failure.
+     * @return bool|PEAR_Error Whether the method is supported or PEAR_Error on failure.
      */
     function hasAuthMech($method)
     {
@@ -673,7 +679,7 @@ class Net_Sieve
      *                           the best (strongest) available method.
      * @param string $euser      The effective uid to authenticate as.
      *
-     * @return void
+     * @return PEAR_Error|null
      */
     function _cmdAuthenticate($uid, $pwd, $userMethod = null, $euser = '')
     {
@@ -698,11 +704,14 @@ class Net_Sieve
             $result = $this->_authEXTERNAL($uid, $pwd, $euser);
             break;
         case 'GSSAPI':
-            $result = $this->_authGSSAPI($pwd);
+            $result = $this->_authGSSAPI();
             break;
         case 'XOAUTH2':
             $result = $this->_authXOAUTH2($uid, $pwd, $euser);
             break;
+        case 'OAUTHBEARER':
+            $result = $this->_authOAUTHBEARER($uid, $pwd, $euser);
+            break;
         default :
             $result = $this->_pear->raiseError(
                 $method . ' is not a supported authentication method'
@@ -715,7 +724,8 @@ class Net_Sieve
             return $res;
         }
 
-        if ($this->_pear->isError($res = $this->_cmdCapability())) {
+        $res = $this->_cmdCapability();
+        if (is_a($res, 'PEAR_Error')) {
             return $this->_pear->raiseError(
                 'Failed to connect, server said: ' . $res->getMessage(), 2
             );
@@ -731,7 +741,7 @@ class Net_Sieve
      * @param string $pass  The password to authenticate with.
      * @param string $euser The effective uid to authenticate as.
      *
-     * @return void
+     * @return PEAR_Error|null
      */
     function _authPLAIN($user, $pass, $euser)
     {
@@ -750,7 +760,7 @@ class Net_Sieve
      *       must have been set.
      * @see  setServicePrincipal()
      *
-     * @return void
+     * @return PEAR_Error|null
      */
     function _authGSSAPI()
     {
@@ -810,7 +820,7 @@ class Net_Sieve
      * @param string $pass  The password to authenticate with.
      * @param string $euser The effective uid to authenticate as. Not used.
      *
-     * @return void
+     * @return PEAR_Error|null
      */
     function _authLOGIN($user, $pass, $euser)
     {
@@ -834,7 +844,7 @@ class Net_Sieve
      * @param string $pass  The password to authenticate with.
      * @param string $euser The effective uid to authenticate as. Not used.
      *
-     * @return void
+     * @return PEAR_Error|null
      */
     function _authCRAMMD5($user, $pass, $euser)
     {
@@ -862,7 +872,7 @@ class Net_Sieve
      * @param string $pass  The password to authenticate with.
      * @param string $euser The effective uid to authenticate as.
      *
-     * @return void
+     * @return PEAR_Error|null
      */
     function _authDigestMD5($user, $pass, $euser)
     {
@@ -892,7 +902,7 @@ class Net_Sieve
         }
 
         if ($this->_toUpper(substr($result, 0, 2)) == 'OK') {
-            return;
+            return null;
         }
 
         /* We don't use the protocol's third step because SIEVE doesn't allow
@@ -912,7 +922,7 @@ class Net_Sieve
      * @param string $pass  The password to authenticate with.
      * @param string $euser The effective uid to authenticate as.
      *
-     * @return void
+     * @return PEAR_Error|null
      *
      * @since 1.1.7
      */
@@ -930,10 +940,11 @@ class Net_Sieve
      * Authenticates the user using the XOAUTH2 method.
      *
      * @param string $user  The userid to authenticate as.
-     * @param string $token The token to authenticate with.
+     * @param string $token The access token prefixed by it's type
+     *                      example: "Bearer $access_token".
      * @param string $euser The effective uid to authenticate as.
      *
-     * @return void
+     * @return PEAR_Error|null
      */
     function _authXOAUTH2($user, $token, $euser)
     {
@@ -946,12 +957,36 @@ class Net_Sieve
         return $this->_sendCmd("AUTHENTICATE \"XOAUTH2\" \"$auth\"");
     }
 
+    /**
+     * Authenticates the user using the OAUTHBEARER method.
+     *
+     * @param string $user  The userid to authenticate as.
+     * @param string $token The access token prefixed by it's type
+     *                      example: "Bearer $access_token".
+     * @param string $euser The effective uid to authenticate as.
+     *
+     * @return PEAR_Error|null
+     *
+     * @see https://www.rfc-editor.org/rfc/rfc7628.html
+     * @since 1.4.7
+     */
+    function _authOAUTHBEARER($user, $token, $euser)
+    {
+        // default to $user if $euser is not set
+        if (! $euser) {
+            $euser = $user;
+        }
+
+        $auth = base64_encode("n,a=$euser,\001auth=$token\001\001");
+        return $this->_sendCmd("AUTHENTICATE \"OAUTHBEARER\" \"$auth\"");
+    }
+
     /**
      * Removes a script from the server.
      *
      * @param string $scriptname Name of the script to delete.
      *
-     * @return boolean  True on success, PEAR_Error otherwise.
+     * @return bool|PEAR_Error True on success, PEAR_Error otherwise.
      */
     function _cmdDeleteScript($scriptname)
     {
@@ -972,7 +1007,7 @@ class Net_Sieve
      *
      * @param string $scriptname Name of the script to retrieve.
      *
-     * @return string  The script if successful, PEAR_Error otherwise.
+     * @return string|PEAR_Error The script if successful, PEAR_Error otherwise.
      */
     function _cmdGetScript($scriptname)
     {
@@ -994,7 +1029,7 @@ class Net_Sieve
      *
      * @param string $scriptname The name of the script to mark as active.
      *
-     * @return boolean  True on success, PEAR_Error otherwise.
+     * @return bool|PEAR_Error True on success, PEAR_Error otherwise.
      */
     function _cmdSetActive($scriptname)
     {
@@ -1013,9 +1048,9 @@ class Net_Sieve
     /**
      * Returns the list of scripts on the server.
      *
-     * @return array  An array with the list of scripts in the first element
-     *                and the active script in the second element on success,
-     *                PEAR_Error otherwise.
+     * @return array|PEAR_Error An array with the list of scripts in the first element
+     *                          and the active script in the second element on success,
+     *                          PEAR_Error otherwise.
      */
     function _cmdListScripts()
     {
@@ -1028,7 +1063,7 @@ class Net_Sieve
             return $res;
         }
 
-        $scripts = array();
+        $scripts = [];
         $activescript = null;
         $res = explode("\r\n", $res);
         foreach ($res as $value) {
@@ -1041,7 +1076,7 @@ class Net_Sieve
             }
         }
 
-        return array($scripts, $activescript);
+        return [$scripts, $activescript];
     }
 
     /**
@@ -1050,7 +1085,7 @@ class Net_Sieve
      * @param string $scriptname Name of the new script.
      * @param string $scriptdata The new script.
      *
-     * @return boolean  True on success, PEAR_Error otherwise.
+     * @return bool|PEAR_Error True on success, PEAR_Error otherwise.
      */
     function _cmdPutScript($scriptname, $scriptdata)
     {
@@ -1077,10 +1112,10 @@ class Net_Sieve
     /**
      * Logs out of the server and terminates the connection.
      *
-     * @param boolean $sendLogoutCMD Whether to send LOGOUT command before
-     *                               disconnecting.
+     * @param bool $sendLogoutCMD Whether to send LOGOUT command before
+     *                            disconnecting.
      *
-     * @return boolean  True on success, PEAR_Error otherwise.
+     * @return bool|PEAR_Error True on success, PEAR_Error otherwise.
      */
     function _cmdLogout($sendLogoutCMD = true)
     {
@@ -1104,7 +1139,7 @@ class Net_Sieve
     /**
      * Sends the CAPABILITY command
      *
-     * @return boolean  True on success, PEAR_Error otherwise.
+     * @return bool|PEAR_Error True on success, PEAR_Error otherwise.
      */
     function _cmdCapability()
     {
@@ -1130,7 +1165,7 @@ class Net_Sieve
     function _parseCapability($data)
     {
         // Clear the cached capabilities.
-        $this->_capability = array('sasl' => array(), 'extensions' => array());
+        $this->_capability = ['sasl' => [], 'extensions' => []];
 
         $data = preg_split('/\r?\n/', $this->_toUpper($data), -1, PREG_SPLIT_NO_EMPTY);
 
@@ -1167,7 +1202,7 @@ class Net_Sieve
      *
      * @param string $cmd The command to send.
      *
-     * @return void
+     * @return PEAR_Error|null
      */
     function _sendCmd($cmd)
     {
@@ -1182,6 +1217,7 @@ class Net_Sieve
             );
         }
         $this->_debug("C: $cmd");
+        return null;
     }
 
     /**
@@ -1189,7 +1225,7 @@ class Net_Sieve
      *
      * @param string $str The string to send.
      *
-     * @return void
+     * @return PEAR_Error|null
      */
     function _sendStringResponse($str)
     {
@@ -1199,7 +1235,7 @@ class Net_Sieve
     /**
      * Receives a single line from the server.
      *
-     * @return string  The server response line.
+     * @return string|PEAR_Error The server response line or PEAR_Error.
      */
     function _recvLn()
     {
@@ -1223,7 +1259,7 @@ class Net_Sieve
     /**
      * Receives a number of bytes from the server.
      *
-     * @param integer $length Number of bytes to read.
+     * @param int $length Number of bytes to read.
      *
      * @return string The server response.
      */
@@ -1242,8 +1278,8 @@ class Net_Sieve
     /**
      * Send a command and retrieves a response from the server.
      *
-     * @param string  $cmd  The command to send.
-     * @param boolean $auth Whether this is an authentication command.
+     * @param string $cmd  The command to send.
+     * @param bool   $auth Whether this is an authentication command.
      *
      * @return string|PEAR_Error Reponse string if an OK response, PEAR_Error
      *                           if a NO response.
@@ -1348,20 +1384,28 @@ class Net_Sieve
      *
      * @param string $userMethod Only consider this method as available.
      *
-     * @return string  The name of the best supported authentication method or
-     *                 a PEAR_Error object on failure.
+     * @return string|PEAR_Error The name of the best supported authentication method or
+     *                           a PEAR_Error object on failure.
      */
     function _getBestAuthMethod($userMethod = null)
     {
         if (!isset($this->_capability['sasl'])) {
             return $this->_pear->raiseError('This server doesn\'t support any authentication methods. SASL problem?');
         }
+
         if (!$this->_capability['sasl']) {
             return $this->_pear->raiseError('This server doesn\'t support any authentication methods.');
         }
 
         if ($userMethod) {
-            if (in_array($userMethod, $this->_capability['sasl'])) {
+            // special case of OAUTH, use the supported method
+            if ($userMethod === 'OAUTH') {
+                foreach (['OAUTHBEARER', 'XOAUTH2'] as $method) {
+                    if (in_array($method, $this->_capability['sasl'])) {
+                        return $method;
+                    }
+                }
+            } elseif (in_array($userMethod, $this->_capability['sasl'])) {
                 return $userMethod;
             }
 
@@ -1386,7 +1430,7 @@ class Net_Sieve
     /**
      * Starts a TLS connection.
      *
-     * @return boolean  True on success, PEAR_Error on failure.
+     * @return bool|PEAR_Error True on success, PEAR_Error on failure.
      */
     function _startTLS()
     {
@@ -1444,14 +1488,14 @@ class Net_Sieve
      *
      * @param string $string A string.
      *
-     * @return integer  The length of the string.
+     * @return int The length of the string.
      */
     function _getLineLength($string)
     {
         if (extension_loaded('mbstring')) {
-            return mb_strlen($string, '8bit');
+            return mb_strlen((string) $string, '8bit');
         } else {
-            return strlen($string);
+            return strlen((string) $string);
         }
     }
 
@@ -1460,13 +1504,13 @@ class Net_Sieve
      *
      * @param string $string The string to convert to lowercase.
      *
-     * @return string  The lowercased string, based on ASCII encoding.
+     * @return string The lowercased string, based on ASCII encoding.
      */
     function _toUpper($string)
     {
         $language = setlocale(LC_CTYPE, 0);
         setlocale(LC_CTYPE, 'C');
-        $string = strtoupper($string);
+        $string = strtoupper((string) $string);
         setlocale(LC_CTYPE, $language);
         return $string;
     }
@@ -1480,6 +1524,8 @@ class Net_Sieve
      */
     function _escape($string)
     {
+        $string = (string) $string;
+
         // Some implementations don't allow UTF-8 characters in quoted-string,
         // use literal-c2s.
         if (preg_match('/[^\x01-\x09\x0B-\x0C\x0E-\x7F]/', $string)) {
@@ -1500,7 +1546,7 @@ class Net_Sieve
     {
         if ($this->_debug) {
             if ($this->_debug_handler) {
-                call_user_func_array($this->_debug_handler, array(&$this, $message));
+                call_user_func_array($this->_debug_handler, [$this, $message]);
             } else {
                 echo "$message\n";
             }
